package com.zulutrade.sample.binaryoptions.trading.client.restclient;

import java.net.URL;

import javax.xml.ws.http.HTTPException;

import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.AuthCache;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.conn.HttpHostConnectException;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.BasicAuthCache;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.zulutrade.sample.binaryoptions.trading.client.configuration.ZuluTradeAuthenticationInfo;
import com.zulutrade.sample.binaryoptions.trading.client.configuration.ZuluTradingServerConfig;
import com.zulutrade.sample.binaryoptions.trading.client.json.ZuluJsonSerializer;
import com.zulutrade.sample.binaryoptions.trading.client.trading.RestClientError;
import com.zulutrade.sample.binaryoptions.trading.client.trading.TradingClientException;

/**
 * Utility class used to send rest calls to a trading server
 * 
 * @author ZuluTrade
 * @version 1.0
 * @since 2015-06-15
 */
public class RestClient {

	public static String sendGetRequest(
		ZuluTradingServerConfig config,
		ZuluTradeAuthenticationInfo authenticationsInfo, 
		String request
	) throws Exception {
		CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(
			AuthScope.ANY,
			new UsernamePasswordCredentials(
				authenticationsInfo.getZuluAccountUsername(), 
				authenticationsInfo.getZuluAccountPassword()
			)
		);
		CloseableHttpClient httpClient = HttpClientBuilder.create().setDefaultCredentialsProvider(credentialsProvider).build();
		HttpGet getRequest = new HttpGet(config.getTradingServicesUrl() + request);
		URL url = new URL(config.getTradingServicesUrl());
		HttpHost host = new HttpHost(url.getHost(), url.getPort(), "http");
		AuthCache authCache = new BasicAuthCache();
		authCache.put(host, new BasicScheme());

		HttpClientContext localContext = HttpClientContext.create();
		localContext.setCredentialsProvider(credentialsProvider);
		localContext.setAuthCache(authCache);

		HttpResponse response = httpClient.execute(host, getRequest, localContext);
		if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
			return EntityUtils.toString(response.getEntity());
		} else {
			throw new HTTPException(response.getStatusLine().getStatusCode());
		}
	}

	public static String get(
		ZuluTradingServerConfig config, 
		ZuluTradeAuthenticationInfo authenticationsInfo, 
		String request
	) throws TradingClientException {
		try {
			return sendGetRequest(config, authenticationsInfo, request);
		} catch (JsonProcessingException jpe) {
			throw new TradingClientException(RestClientError.HTTP_BODY_SERIALIZATION_ERROR);
		} catch (HTTPException hte) {
			int statusCode = ((HTTPException) hte).getStatusCode();
			if (statusCode == HttpStatus.SC_UNAUTHORIZED) {
				throw new TradingClientException(RestClientError.AUTHENTICATION_ERROR);
			} else if (statusCode == HttpStatus.SC_FORBIDDEN) {
				throw new TradingClientException(RestClientError.AUTHORIZATION_ERROR);
			} else if (statusCode == HttpStatus.SC_NOT_FOUND) {
				throw new TradingClientException(RestClientError.UNAVAILABLE_RESOURCE_ERROR);
			}
			throw new TradingClientException(RestClientError.REST_CLIENT_DEFAULT_ERROR);
		} catch (HttpHostConnectException hce) {
			throw new TradingClientException(RestClientError.CONNECTION_ERROR);
		} catch (Exception e) {
			throw new TradingClientException(RestClientError.REST_CLIENT_DEFAULT_ERROR);
		}
	}

	public static String sendPostRequest(
		ZuluTradingServerConfig config,
		ZuluTradeAuthenticationInfo authenticationsInfo, 
		String request,
		String jsonBody
	) throws Exception {
		CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
		credentialsProvider.setCredentials(
			AuthScope.ANY,
			new UsernamePasswordCredentials(
				authenticationsInfo.getZuluAccountUsername(), 
				authenticationsInfo.getZuluAccountPassword()
			)
		);
		CloseableHttpClient httpClient = HttpClientBuilder.create().setDefaultCredentialsProvider(credentialsProvider).build();
		HttpPost postRequest = new HttpPost(config.getTradingServicesUrl()+ request);
		StringEntity input = new StringEntity(jsonBody);
		input.setContentType("application/json");
		postRequest.setEntity(input);
		URL url = new URL(config.getTradingServicesUrl());
		HttpHost host = new HttpHost(url.getHost(), url.getPort(), "http");
		AuthCache authCache = new BasicAuthCache();
		authCache.put(host, new BasicScheme());

		HttpClientContext localContext = HttpClientContext.create();
		localContext.setCredentialsProvider(credentialsProvider);
		localContext.setAuthCache(authCache);

		HttpResponse response = httpClient.execute(host, postRequest, localContext);
		if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
			return EntityUtils.toString(response.getEntity());
		} else {
			throw new HTTPException(response.getStatusLine().getStatusCode());
		}
	}

	public static String post(
		ZuluTradingServerConfig config,
		ZuluTradeAuthenticationInfo authenticationsInfo, 
		String request,
		Object body
	) throws TradingClientException {
		try {
			String bodyStr = ZuluJsonSerializer.jsonSerialize(body);
			return sendPostRequest(config, authenticationsInfo, request, bodyStr);
		} catch (JsonProcessingException jpe) {
			throw new TradingClientException(RestClientError.HTTP_BODY_SERIALIZATION_ERROR);
		} catch (HTTPException hte) {
			int statusCode = ((HTTPException) hte).getStatusCode();
			if (statusCode == HttpStatus.SC_UNAUTHORIZED) {
				throw new TradingClientException(RestClientError.AUTHENTICATION_ERROR);
			} else if (statusCode == HttpStatus.SC_FORBIDDEN) {
				throw new TradingClientException(RestClientError.AUTHORIZATION_ERROR);
			} else if (statusCode == HttpStatus.SC_NOT_FOUND) {
				throw new TradingClientException(RestClientError.UNAVAILABLE_RESOURCE_ERROR);
			}
			throw new TradingClientException(RestClientError.REST_CLIENT_DEFAULT_ERROR);
		} catch (HttpHostConnectException hce) {
			throw new TradingClientException(RestClientError.CONNECTION_ERROR);
		} catch (Exception e) {
			throw new TradingClientException(RestClientError.REST_CLIENT_DEFAULT_ERROR);
		}
	}

}
